VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "License"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'Member-Variablen - Mssen ber Eigenschaften (Properties) ausgelesen und/oder geschrieben werden
Private m_LastDate As Date
Private m_Checksum As Long

' ffentliche Variablen - Knnen frei ausgelesen und geschrieben werden
Public StartDate As Date
Public EndDate As Date
Public OneSystemOnly As Boolean

Public Enum LicenseInfo
   LicenseOkay = 0         ' Alles in Ordnung
   FileManipulated = 1     ' Die Datei wurde verndert
   DateManipulated = 2     ' Die Windows-Uhr wurde verstellt
   LicenseExpired = 3      ' Die Lizenz ist ausgelaufen
   LicenseNotBegun = 4     ' Die Lizenz hat noch nicht angefangen
   WrongApplication = 5    ' Die Lizenz gilt fr eine andere Anwendung (falsches Passwort)
   FileNotExisting = 6     ' Die Lizenz-Datei konnte nicht gefunden werden
End Enum

' kleiner Trick, damit die Namen der Konstanten im Enum ihre Schreibweise beibehalten
#If False Then
   Dim LicenseOkay, FileManipulated, DateManipulated, LicenseExpired, LicenseNotBegun
   Dim FunctionManipulated, WrongApplication
#End If

' Das letzte Datum darf nicht verndert werden!
Property Get LastDate() As Date
   LastDate = m_LastDate
End Property

' Das Passwort muss nur festgelegt, nicht aber ausgelesen werden
Property Let PassWord(RHS As String)
   m_Checksum = CheckSum(StrConv(RHS, vbFromUnicode))
End Property

' Schreibt eine Lizenz in eine Datei
Public Sub WriteTo(FileName As String, Optional CPUID As String)
   Dim PropBag As New PropertyBag, FF As Byte, BA() As Byte
   
   'Alle relevanten Daten in die PropertyBag schreiben
   Call PropBag.WriteProperty("Start", StartDate)
   Call PropBag.WriteProperty("End", EndDate)
   Call PropBag.WriteProperty("Last", Date)
   
   'den Inhalt zwischenspeichern
   BA = NSGEncode(PropBag.Contents)
   Set PropBag = New PropertyBag
   
   'Wieder in die PropertyBag schreiben
   Call PropBag.WriteProperty("Contents", BA)
   Call PropBag.WriteProperty("Decode", m_Checksum)
   Call PropBag.WriteProperty("Check", CheckSum(BA))
   
   'finales Zwischenspeichern
   BA = PropBag.Contents
   
   'Datei vorsichtshalber lschen...
   On Error Resume Next
   Call Kill(FileName)
   On Error GoTo 0
   
   'Dateizugriffsnummer holen
   FF = FreeFile
   'Und speichern
   Open FileName For Binary As #FF
      Put #FF, , BA
   Close #FF
End Sub

' Liest eine Lizenz aus einer Datei
Public Function ReadFrom(FileName As String) As LicenseInfo
   Dim PropBag As New PropertyBag, FF As Byte
   Dim BA() As Byte  ' Muss leider sein
   
   'Testen, ob die Datei existiert
   If Dir(FileName) = "" Then
      ReadFrom = FileNotExisting
      Exit Function
   End If
   
   'Dateizugriffsnummer holen
   FF = FreeFile
   'und die Datei auslesen
   Open FileName For Binary As #FF
      ReDim BA(LOF(FF))
      Get #FF, , BA
   Close #FF
   
   'Die Zuweisung ergibt einen Fehler, wenn die Datei manipuliert wurde
   On Error GoTo Err_FileInvalid
   PropBag.Contents = BA
   On Error GoTo 0
   
   'gespeicherte Prfsumme mit der tatschlichen vergleichen
   If PropBag.ReadProperty("Check") <> CheckSum(PropBag.ReadProperty("Contents")) Then
      ReadFrom = FileManipulated
      Exit Function
   End If
   
   'Wenn Prfsumme des aktuellen Passwortes nicht mit der der Lizenz bereinstimmt,
   'gilt die Lizenz nicht fr die aktuelle Anwendung
   If Not m_Checksum = PropBag.ReadProperty("Decode") Then
      ReadFrom = WrongApplication
      Exit Function
   End If
   
   BA = NSGDecode(PropBag.ReadProperty("Contents"))
   
   'Die Zuweisung ergibt einen Fehler, wenn die Datei manipuliert wurde
   On Error GoTo Err_FileInvalid
   PropBag.Contents = BA
   On Error GoTo 0
   
   'Alle relevanten Daten auslesen
   m_LastDate = PropBag.ReadProperty("Last")
   StartDate = PropBag.ReadProperty("Start")
   EndDate = PropBag.ReadProperty("End")
   
   'Wenn die Differenz zwischen aktuellem Datum und dem Datum
   'des letzten Verwendens negativ ist, wurde an der Uhr gedreht
   If DateDiff("d", m_LastDate, Date) < 0 Then
      ReadFrom = DateManipulated
      Exit Function
   End If
   
   'Wenn die Differenz zwischen aktuellem Datum und Startdatum der
   'Lizenz negativ ist, hat die Lizenz noch nicht begonnen
   If DateDiff("d", StartDate, Date) < 0 Then
      ReadFrom = LicenseNotBegun
      Exit Function
   End If
   
   'Wenn die Differenz zwischen aktuellem Datum und Enddatum der
   'Lizenz positiv ist, ist die Lizenz abgelaufen
   If DateDiff("d", EndDate, Date) > 0 Then
      ReadFrom = LicenseExpired
      Exit Function
   End If
   
   'Wenn bis hierher ausgefhrt wurde, ist die Lizenz in Ordnung
   '(Oder die Funktion manipuliert ;-) )
   ReadFrom = LicenseOkay
   
   'Funktion verlassen
   Exit Function
   
   'Wenn Ein Fehler aufgetreten ist:
Err_FileInvalid:
   ReadFrom = FileManipulated
End Function

' Entschlsselung
Private Function NSGDecode(Expression() As Byte) As Byte()
   Dim n As Long, Temp As Long, BA() As Byte
   
   Call Rnd(-1)
   Call Randomize(m_Checksum)
   
   ReDim BA(UBound(Expression))
   
   For n = 0 To UBound(Expression)
      Temp = Expression(n) - Int(Rnd() * 256)
      If Temp < 0 Then Temp = Temp + 256
      BA(n) = Temp
   Next n
   
   NSGDecode = BA
End Function

' Verschlsselung
Private Function NSGEncode(Expression() As Byte) As Byte()
   Dim n As Long, BA() As Byte
   
   Call Rnd(-1)
   Call Randomize(m_Checksum)
   
   ReDim BA(UBound(Expression))
   
   For n = 0 To UBound(Expression)
      BA(n) = (Expression(n) + Int(Rnd() * 256)) And &HFF
   Next n
   
   NSGEncode = BA
End Function

' Berechnen einer Prfsumme (eigener Algorithmus, kann angepasst werden)
Private Function CheckSum(FromText() As Byte) As Long
   Dim n As Long
   
   For n = 0 To UBound(FromText)
      CheckSum = (CheckSum + FromText(n) * (2 - Cos(n) * _
                  FromText(n) ^ (1 / (n + 1)))) And &HFFFFFFFF
   Next n
End Function
